@@ -15,6 +15,9 @@ import shlex
 import shutil
 from cStringIO import StringIO
 
+from itertools import chain, imap
+
+
 from setuptools import Command
 from distutils.util import convert_path
 from distutils import log
@@ -39,8 +42,13 @@ from py2app.filters import \
     not_stdlib_filter, not_system_filter, has_filename_filter
 from py2app import recipes
 
+from distutils.sysconfig import get_config_var
+PYTHONFRAMEWORK=get_config_var('PYTHONFRAMEWORK')
+
 
 def get_zipfile(dist):
+    if sys.version_info[0] == 3:
+        return "python%d%d.zip"%(sys.version_info[:2])
     return getattr(dist, "zipfile", None) or "site-packages.zip"
 
 def framework_copy_condition(src):
@@ -378,12 +386,19 @@ class py2app(Command):
             version = sys.version
         version = version[:3]
         info = None
+        if os.path.exists(os.path.join(prefix, ".Python")):
+            # We're in a virtualenv environment, locate the real prefix
+            fn = os.path.join(prefix, "lib", "python%d.%d"%(sys.version_info[:2]), "orig-prefix.txt")
+            if os.path.exists(fn):
+                prefix = open(fn, 'rU').read().strip()
+
         try:
             fmwk = macholib.dyld.framework_find(prefix)
         except ValueError:
             info = None
         else:
             info = macholib.dyld.framework_info(fmwk)
+
         if info is not None:
             dylib = info['name']
             runtime = os.path.join(info['location'], info['name'])
@@ -613,7 +628,7 @@ class py2app(Command):
             dgraph = os.path.join(appdir, appname + '.html')
             print ("*** creating dependency html: %s ***"
                 % (os.path.basename(dgraph),))
-            mf.create_xref(file(dgraph, 'w'))
+            mf.create_xref(open(dgraph, 'w'))
 
     def build_graph(self, mf, flatpackages):
         for target in self.targets:
@@ -623,7 +638,7 @@ class py2app(Command):
             dgraph = os.path.join(appdir, appname + '.dot')
             print ("*** creating dependency graph: %s ***"
                 % (os.path.basename(dgraph),))
-            mf.graphreport(file(dgraph, 'w'), flatpackages=flatpackages)
+            mf.graphreport(open(dgraph, 'w'), flatpackages=flatpackages)
 
     def finalize_modulefinder(self, mf):
         py_files, extensions = parse_mf_results(mf)
@@ -747,7 +762,9 @@ class py2app(Command):
         arcname = self.make_lib_archive(archive_name,
             base_dir=self.collect_dir, verbose=self.verbose,
             dry_run=self.dry_run)
-        self.lib_files.append(arcname)
+
+        # XXX: this doesn't work with python3
+        #self.lib_files.append(arcname)
 
         # build the executables
         for target in self.targets:
@@ -758,7 +775,20 @@ class py2app(Command):
             if self.semi_standalone:
                 self.symlink(sys.executable, execdst)
             else:
-                self.copy_file(sys.executable, execdst)
+                if os.path.exists(os.path.join(sys.prefix, ".Python")):
+                    fn = os.path.join(sys.prefix, "lib", "python%d.%d"%(sys.version_info[:2]), "orig-prefix.txt")
+                    if os.path.exists(fn):
+                        prefix = open(fn, 'rU').read().strip()
+
+                    rest_path = sys.executable[len(sys.prefix)+1:]
+                    if rest_path.startswith('.'):
+                        rest_path = rest_path[1:]
+
+                    print "XXXX", os.path.join(prefix, rest_path)
+                    self.copy_file(os.path.join(prefix, rest_path), execdst)
+
+                else:
+                    self.copy_file(sys.executable, execdst)
             if not self.debug_skip_macholib:
                 mm = PythonStandalone(self, dst, executable_path=exp)
                 dylib, runtime = self.get_runtime()
@@ -887,7 +917,7 @@ class py2app(Command):
 
     def copy_framework(self, info, dst):
         force, self.force = self.force, True
-        if info['shortname'] == 'Python':
+        if info['shortname'] == PYTHONFRAMEWORK:
             self.copy_python_framework(info, dst)
         else:
             self.copy_versioned_framework(info, dst)
@@ -1050,7 +1080,7 @@ class py2app(Command):
         if not isinstance(bootstrap, basestring):
             return bootstrap.getvalue()
         else:
-            return file(bootstrap, 'rU').read()
+            return open(bootstrap, 'rU').read()
 
     def create_pluginbundle(self, target, script, use_runtime_preference=True):
         base = target.get_dest_base()
@@ -1115,7 +1145,12 @@ class py2app(Command):
 
         # symlink python executable
         execdst = os.path.join(appdir, 'Contents', 'MacOS', 'python')
-        self.symlink(sys.executable, execdst)
+        prefixPathExecutable = os.path.join(sys.prefix, 'bin', 'python')
+        if os.path.exists(prefixPathExecutable):
+            pyExecutable = prefixPathExecutable
+        else:
+            pyExecutable = sys.executable
+        self.symlink(pyExecutable, execdst)
 
         # make PYTHONHOME
         pyhome = os.path.join(resdir, 'lib', 'python' + sys.version[:3])
@@ -1152,7 +1187,7 @@ class py2app(Command):
         aliasdata = FSRef(script).FSNewAliasMinimal().data
 
         bootfn = '__boot__'
-        bootfile = file(os.path.join(resdir, bootfn + '.py'), 'w')
+        bootfile = open(os.path.join(resdir, bootfn + '.py'), 'w')
         for fn in target.prescripts:
             bootfile.write(self.get_bootstrap_data(fn))
             bootfile.write('\n\n')
@@ -1183,7 +1218,7 @@ class py2app(Command):
         self.compile_mappingmodels(resdir)
 
         bootfn = '__boot__'
-        bootfile = file(os.path.join(resdir, bootfn + '.py'), 'w')
+        bootfile = open(os.path.join(resdir, bootfn + '.py'), 'w')
         for fn in target.prescripts:
             bootfile.write(self.get_bootstrap_data(fn))
             bootfile.write('\n\n')
@@ -1192,6 +1227,10 @@ class py2app(Command):
 
         self.copy_file(script, resdir)
         pydir = os.path.join(resdir, 'lib', 'python' + sys.version[:3])
+        if sys.version_info[0] == 2:
+            arcdir = os.path.join(resdir, 'lib', 'python' + sys.version[:3])
+        else:
+            arcdir = os.path.join(resdir, 'lib')
         realhome = os.path.join(sys.prefix, 'lib', 'python' + sys.version[:3])
         self.mkpath(pydir)
         self.symlink('../../site.py', os.path.join(pydir, 'site.py'))
@@ -1214,7 +1253,11 @@ class py2app(Command):
                 sys.version[:3])), os.path.join(inc_dir, 'pyconfig.h'))
 
 
-        self.copy_file(arcname, pydir)
+        self.copy_file(arcname, arcdir)
+        if sys.version_info[0] != 2:
+            import zlib
+            self.copy_file(zlib.__file__, os.path.dirname(arcdir))
+        
         ext_dir = os.path.join(pydir, os.path.basename(self.ext_dir))
         self.copy_tree(self.ext_dir, ext_dir, preserve_symlinks=True)
         self.copy_tree(self.framework_dir,
